/* data_flow.c 
 * Nick Jones
 * Created 4/5/19 
 * Modified 4/5/19 */

/* ESP Commands:
 *  0x10 - nothing changed
 *  0x11 - start recording
 *  0x12 - stop recording
 *  0x13 - change gain
 *  0x14 - download data
 *  From the PIC: 0x20 - update the PIC
 */

#include "bat_news_travels_fast.h"

/* initializeSPI: set config bits for SPI2 (SD card) and SPI2 (ESP) */
void initSPI() {
    // setup ESP interface
    SPI3CONbits.MSTEN = 1;  // enable PIC as master
    SPI3CONbits.SMP = 0;    // sample at the middle
    SPI3CONbits.CKP = 0;    // clock idle state low
    SPI3CONbits.CKE = 1;    // Output on falling edge
    SPI3CONbits.MODE16 = 0; // use 8 bit transfer mode
    SPI3CONbits.MODE32 = 0;
    SPI3BRG = 29;           // set clock to 1 MHz
    SPI3CONbits.ON = 1;     // enable SPI
 
    _TRISD10 = 0;            // set SS pin as output
    ESP_PORT = 1;
}

/* initDAC */
void initDAC() {
    I2C5BRG = 0x124;         // set BAUD rate to 100 kHz
    I2C5CONbits.I2CEN = 1;  // enable I2C
}

/* startI2C */
void startI2C() {
    I2C5CONbits.SEN = 1;
    while (I2C5CONbits.SEN);
}

/* stopI2C */
void stopI2C() {
    I2C5CONbits.PEN = 1;
    while (I2C5CONbits.PEN);
}

/* putI2C: writes data and returns ACK bit */
int putI2C(byte data) {
    I2C5TRN = data;
    while (I2C5STATbits.TRSTAT);
    
    return I2C5STATbits.ACKSTAT;
}

/* ESPWrite: write byte to ESP and return the response */
byte ESPWrite(byte data) {
    //if (SPI3STATbits.SPITBE)
    SPI3BUF = data;
    
    while (!SPI3STATbits.SPIRBF);
    
    return SPI3BUF;
}

/* changeGain: write new value to DAC */
int changeGain(byte dbGain) {
    double v_gain;
    uint32 dac_gain;
    
    // compute DAC input for specified gain dB
    dbGain = (dbGain > 80) ? 80 : dbGain;
    v_gain = 1.1 - ((double)dbGain / 80);
    dac_gain = v_gain / 3.3 * 1023;
    
    // write to DAC
    byte dac_address = 0b11000000;
    byte config_byte = 0b01000000;
    
    startI2C();
    putI2C(dac_address & 0xFE);     // device address
    putI2C(config_byte);             // configuration bits
    putI2C((byte)(dac_gain >> 2));      // 8 MSBs of gain
    putI2C((byte)(dac_gain << 6));      // 2 LSBs of gain
    stopI2C();
}

/* handleServerUpdate: take care of user input from ESP */
int handleServerUpdate(byte status, uint32 data) {
    switch (status) {
        case START_REC:
            if (deviceMode != MODE_RECORD && deviceMode != MODE_DOWNLOAD)
                deviceMode = MODE_RECORD;
            break;
        case STOP_REC:
            if (deviceMode == MODE_RECORD)
                deviceMode = MODE_AWAKE;
            break;
        case CHANGE_GAIN:
            changeGain((byte)data);
            break;
        case DOWNLOAD:
            break;
        default:
            break;
    }
    
    return 0;
}