/* interrupts.c 
 * Nick Jones
 * Created 4/5/19 
 * Modified 4/5/19 */

#include "bat_news_travels_fast.h"

/* initializeTimers: setup timer to read ESP */
void initTimers() {
    // T1 controls SD card status
    T1CONbits.TON = 0;      // turn timer off
    T1CONbits.SIDL = 0;     // continue timer in idle mode
    T1CONbits.TGATE = 0;    // disable gate accumulation
    T1CONbits.TCKPS = 0b11; // set prescaler to 8
    
    TMR1 = 0;               // reset timer value
    PR1 = 6563;            // set frequency to 1 kHz
    
    // T2 controls web server updates
    T2CONbits.ON = 0;      // turn timer off
    T2CONbits.SIDL = 0;     // continue timer in idle mode
    T2CONbits.T32 = 0;      // operate in 16 bit mode
    T2CONbits.TGATE = 0;    // disable gate accumulation
    T2CONbits.TCKPS = 0b11; // set prescaler to 8
    T2CON &= 0xFFFC;
    
    TMR2 = 0;               // reset timer value
    PR2 = 750000;            // set frequency to 10 Hz
    
    T3CONbits.ON = 0;
    T3CONbits.SIDL = 0;
    T3CONbits.TGATE = 0;
    T3CONbits.TCKPS = 0b000;
    T3CON &= 0xFFFC;
    
    TMR3 = 0;
    PR3 = SAMPLING_PR;        // sample at 250 kHz
    
    T1CONbits.ON = 1;       // turn timers on
    T2CONbits.ON = 1;
}

/* initializeInterrupts: setup interrupts */
void initInterrupts() {
    __builtin_enable_interrupts();  // enable interrupts globally    
    INTCONbits.MVEC = 1;    // enable multi-vectored interrupts
    
    IPC2bits.T2IP = 2;      // T2 is used for updated web server status
    IFS0bits.T2IF = 0;      // clear flag
    IEC0bits.T2IE = 1;      // enable interrupt
    
    IPC1bits.T1IP = 3;      // T1 is used for updating the SD card status
    IFS0bits.T1IF = 0;      // clear flag
    IEC0bits.T1IE = 1;      // enable interrupt
    
    IPC3bits.T3IP = 1;      // T3 used for ADC and for SD write buffering
    IFS0bits.T3IF = 0;
    // IEC0bits.T3IE = 1;   // don't enable interrupt till done writing to SD card
    
    T1CONbits.TCS = 0;
    CNCONbits.ON = 1;
    CNENbits.CNEN0 = 1;     // change notification interrupt for button
    IPC6bits.CNIP = 4;
    IFS1bits.CNIF = 0;
    IEC1bits.CNIE = 1;
    
    IPC6bits.AD1IP = 6;     // ADC conversion interrupt
    IFS1bits.AD1IF = 0;
    //IEC1bits.AD1IE = 1;
}

/* Timer2 interrupt handler */
void __ISR(TIMER2_VEC_NUMBER, IPL2AUTO) Timer2Interrupt() {
    ESP_PORT = 0;               // start SPI communication with the ESP
    
    int i;
    byte status = 0;
    uint32 status_data = 0;
    
    ESPWrite(0x04);             // send "get status" command to ESP
    status = ESPWrite(0);       // receive response
    for (i = 0; i < 3; i++) {
        status_data |= ((uint32)ESPWrite(0) << i*8);
    }
    
    /*ESPWrite(0x02);
    ESPWrite(0x00);
    ESPWrite(deviceMode);
    for (i = 0; i < 31; i++)
        ESPWrite(0x00);*/
       
    ESP_PORT = 1;               // end SPI communication
    
    if (status != NO_STATUS)
        handleServerUpdate(status, status_data);

   // _RD2 = (_RD2) ? 0 : 1;
    IFS0bits.T2IF = 0;      // clear flag 
}

/* T3 Interrupt Handler */
void __ISR(TIMER3_VEC_NUMBER, IPL1AUTO) Timer3Interrupt() {
    // if timer is being used to give SD card time to flash memory
    if (sdWaiting == 1) {
        sdWaiting = 0;          // done waiting
    
        PR3 = SAMPLING_PR;      // reset timer period
        IEC0bits.T3IE = 0;      // disable itself
        T3CONbits.ON = 0;
    }
    // otherwise it's being used to blink the status LED
    else {
        _RB13 = (_RB13) ? 0 : 1;
    }
    
    IFS0bits.T3IF = 0;      // clear flag
}

/* CN Interrupt Handler */
void __ISR(CN_VEC_NUMBER, IPL3AUTO) CNInterrupt() {
    // if the button is pressed, start or stop recording
    if (_RC14 == 0) {
        if (deviceMode == MODE_RECORD || deviceMode == MODE_ERROR)
            deviceMode = MODE_AWAKE;
        else if (deviceMode == MODE_AWAKE)
            deviceMode = MODE_RECORD;
    }
    
    IFS1bits.CNIF = 0;          // clear the flag
}

/* ADC Interrupt Handler */
void __ISR(ADC_VEC_NUMBER, IPL6AUTO) ADCInterrupt() {
    if (AD1CON2bits.BUFS) {     // ADC is filling words 0x8 - 0xF
        pADCBuffer[ADCBufferCounter]   = (ADC1BUF0);
        pADCBuffer[ADCBufferCounter+1] = (ADC1BUF0 >> 8) | (ADC1BUF1 << 2);
        pADCBuffer[ADCBufferCounter+2] = (ADC1BUF1 >> 6) | (ADC1BUF2 << 4);
        pADCBuffer[ADCBufferCounter+3] = (ADC1BUF2 >> 4) | (ADC1BUF3 << 6);
        pADCBuffer[ADCBufferCounter+4] = (ADC1BUF3 >> 2);
        pADCBuffer[ADCBufferCounter+5] = (ADC1BUF4);
        pADCBuffer[ADCBufferCounter+6] = (ADC1BUF4 >> 8) | (ADC1BUF5 << 2);
        pADCBuffer[ADCBufferCounter+7] = (ADC1BUF5 >> 6) | (ADC1BUF6 << 4);
        pADCBuffer[ADCBufferCounter+8] = (ADC1BUF6 >> 4) | (ADC1BUF7 << 6);
        pADCBuffer[ADCBufferCounter+9] = (ADC1BUF7 >> 2);
    }
    else {
        pADCBuffer[ADCBufferCounter]   = (ADC1BUF8);
        pADCBuffer[ADCBufferCounter+1] = (ADC1BUF8 >> 8) | (ADC1BUF9 << 2);
        pADCBuffer[ADCBufferCounter+2] = (ADC1BUF9 >> 6) | (ADC1BUFA << 4);
        pADCBuffer[ADCBufferCounter+3] = (ADC1BUFA >> 4) | (ADC1BUFB << 6);
        pADCBuffer[ADCBufferCounter+4] = (ADC1BUFB >> 2);
        pADCBuffer[ADCBufferCounter+5] = (ADC1BUFC);
        pADCBuffer[ADCBufferCounter+6] = (ADC1BUFC >> 8) | (ADC1BUFD << 2);
        pADCBuffer[ADCBufferCounter+7] = (ADC1BUFD >> 6) | (ADC1BUFE << 4);
        pADCBuffer[ADCBufferCounter+8] = (ADC1BUFE >> 4) | (ADC1BUFF << 6);
        pADCBuffer[ADCBufferCounter+9] = (ADC1BUFF >> 2);
    }
    
    ADCBufferCounter += 10;
    
    IFS1bits.AD1IF = 0;         // clear interrupt flag
}